class FilterArgs:
    _list_args = ("subjectId", "parentSubjectId", "parentSubject", "brandId")

    def __init__(self, str_args):
        self.str_args = str_args
        self.subject_id = ""
        self.parent_subject_id = ""
        self.parent_subject = ""
        self.brand_id = ""

        self._flag_subject_id = "subjectId"
        self._flag_parent_subject_id = "parentSubjectId"
        self._flag_parent_subject = "parentSubject"
        self._flag_brand_id = "brandId"

        self.parse_args()

    def parse_args(self):
        if self.str_args:
            if self._check_subject_id():
                self._set_subject_id()
            if self._check_parent_subject_id():
                self._set_parent_subject_id()
            if self._check_parent_subject():
                self._set_parent_subject()
            if self._check_brand_id():
                self._set_brand_id()

    def _get_arg(self, key) -> str:
        """в этой функции достаем аргумент по флагу"""
        result = ""

        if self.str_args.startswith(key):
            result = self.str_args.split(":")[1]

        return result

    def _check_subject_id(self) -> bool:
        return self._flag_subject_id in self.str_args

    def _set_subject_id(self):
        self.subject_id = self._get_arg(self._flag_subject_id)

    def _check_parent_subject_id(self) -> bool:
        return self._flag_parent_subject_id in self.str_args

    def _set_parent_subject_id(self):
        self.parent_subject_id = self._get_arg(self._flag_parent_subject_id)

    def _check_parent_subject(self) -> bool:
        return self._flag_parent_subject in self.str_args

    def _set_parent_subject(self):
        self.parent_subject = self._get_arg(self._flag_parent_subject).replace("\\", '\\"')

    def _check_brand_id(self) -> bool:
        return self._flag_brand_id in self.str_args

    def _set_brand_id(self):
        self.brand_id = self._get_arg(self._flag_brand_id)

    def __str__(self):
        return self.del_space(self.str_args)

    def get_result_for_write(self):
        # TODO: супер костыли, удаляем пробел и @sort с конца строки, нужно передалть
        return self.del_sort(self.del_space(self.str_args)).replace("\\", '\\"')
        """пока проблемы с этой функцией, нужно подумать как обрабатывать когда несколько аргументов в фильтре
        кейс не частый, но такие есть"""
        if self.str_args is None:
            return ""

        list_result = []
        if self.parent_subject_id:
            list_result.append(f"{self._flag_parent_subject_id}:{self.del_space(self.parent_subject_id)}")
        if self.parent_subject:
            if self.parent_subject_id:
                pass
            else:
                list_result.append(f"{self._flag_parent_subject}:{self.del_space(self.parent_subject)}")
        if self.subject_id:
            list_result.append(f"{self._flag_subject_id}:{self.del_space(self.subject_id)}")
        if self.brand_id:
            list_result.append(f"{self._flag_brand_id}:{self.del_space(self.brand_id)}")
        return " OR ".join(list_result)

    def del_space(self, arg):
        if arg[-1:] == " ":
            return arg[:-1]
        return arg

    def del_sort(self, arg):
        if arg[-7:] == " @sort:":
            return arg[:-7]
        return arg

    def check_filter_args(self):
        """функция возвращает tuple с аргументами фильтра
        пример: ("brandId",) или ("brandId","subjectId",)"""
        pass



from wbxSearch import WorkCSV, Extradition


class WorkExtradition:
    def __init__(self, path_read_file: str, path_write_file: str = ""):
        self.csv = WorkCSV(path_file=path_read_file)
        self._data_csv = self.csv.read_csv()
        self.path_write_file = path_write_file
        self._list_ex = [Extradition(x) for x in self._data_csv]

        """в этот лист записываются результаты поиска выдач, потом из этого листа
        # формируем этот файл, либо удалям эти выдачи из листа list_ex"""
        self._new_list_ex = []

        if self.path_write_file:
            self._new_csv = WorkCSV(path_file=path_write_file)

    def _param_search(self, param: str,
                      value_param: str,
                      complete_match: bool = True,
                      contains: bool = False) -> list:
        """
        ищем записи по параметру в выдачах
        :param param: наименование параметра по которому ищем(всего их 10, список в шапке класса Extradition)
        :param value_param: значение параметра по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет value_param(True - ищем где нет value_param, по умолчанию False)
        :return: list с подходящими выдачами
        """
        # поиск по полному совпадению
        if complete_match and contains is False:
            result_list = [x for x in self._list_ex if value_param == x.get_param(param)]
        # поиск по частичному совпадению
        elif complete_match is False and contains is False:
            result_list = [x for x in self._list_ex if value_param in x.get_param(param)]
        # попадают в результат все выдачи где нет полного совпадения
        elif complete_match and contains:
            result_list = [x for x in self._list_ex if value_param != x.get_param(param)]
        # попадают в результат все выдачи где нет частичного совпадения
        elif complete_match is False and contains:
            result_list = [x for x in self._list_ex if value_param not in x.get_param(param)]
        else:
            raise "Не правильно выставлены условия"

        self._new_list_ex += result_list
        return result_list

    def search_query_search(self, search_query: str, complete_match: bool = True, contains: bool = False) -> list:
        """
        ищем записи по 1 столбцу в выдачах
        :param search_query: наименование search_query по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет search_query(True - ищем где нет value_param, по умолчанию False)
        :return: list с подходящими выдачами
        """
        return self._param_search("search_query", search_query, complete_match, contains)

    def search_preset_id(self, preset_id: str, complete_match: bool = True, contains: bool = False) -> list:
        """
        ищем записи по 2 столбцу в выдачах
        :param preset_id: наименование preset_id по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет preset_id(True - ищем где нет value_param, по умолчанию False)
        :return: list с подходящими выдачами
        """
        return self._param_search("preset_id", preset_id, complete_match, contains)

    def active_search(self, active: bool = True) -> list:
        """ищем записи по 3 столбцу в выдачах, это поле актив, оно бывает двух видов yes и no,
        по умолчанию ищем yes"""
        result_list = [x for x in self._list_ex if x.get_active() is active]
        self._new_list_ex += result_list
        return result_list

    def kind_search(self, kind: str, complete_match: bool = True, contains: bool = False) -> list:
        """
        ищем записи по 4 столбцу в выдачах
        :param kind: наименование kind по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет kind(True - ищем где нет kind, по умолчанию False)
        :return: list с подходящими выдачами
        """
        return self._param_search("kind", kind, complete_match, contains)

    def parent_search(self, parent: str, complete_match: bool = True, contains: bool = False) -> list:
        """
        ищем записи по 5 столбцу в выдачах
        :param parent: наименование parent по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет parent(True - ищем где нет parent, по умолчанию False)
        :return: list с подходящими выдачами
        """
        return self._param_search("parent", parent, complete_match, contains)

    def miner_search(self, miner: str, complete_match: bool = True, contains: bool = False) -> list:
        """
        ищем записи по 6 столбцу в выдачах
        :param miner: наименование parent по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет miner(True - ищем где нет miner, по умолчанию False)
        :return: list с подходящими выдачами
        """
        return self._param_search("miner", miner, complete_match, contains)

    def miners_args_search(self, miners_args: str, complete_match: bool = True, contains: bool = False) -> list:
        """
        ищем записи по 7 столбцу в выдачах
        :param miners_args: наименование parent по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет miners_args(True - ищем где нет miners_args, по умолчанию False)
        :return: list с подходящими выдачами
        """
        # TODO miners_args это самостоятельный объект с разными параметрами, данная функция осуществляет работу с
        #  miners_args как с строкой, нужна дополнительная функция по поиску по объекту miners_args
        return self._param_search("miners_args", miners_args, complete_match, contains)

    def shard_kind_search(self, shard_kind: str, complete_match: bool = True, contains: bool = False) -> list:
        """
        ищем записи по 8 столбцу в выдачах
        :param shard_kind: наименование parent по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет shard_kind(True - ищем где нет shard_kind, по умолчанию False)
        :return: list с подходящими выдачами
        """
        return self._param_search("shard_kind", shard_kind, complete_match, contains)

    def query_search(self, query: str, complete_match: bool = True, contains: bool = False) -> list:
        """
        ищем записи по 9 столбцу в выдачах
        :param query: наименование parent по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет query(True - ищем где нет query, по умолчанию False)
        :return: list с подходящими выдачами
        """
        return self._param_search("query", query, complete_match, contains)

    def category_search(self, category: str, complete_match: bool = True, contains: bool = False) -> list:
        """
        ищем записи по 10 столбцу в выдачах
        :param category: наименование parent по которому ищем
        :param complete_match: полное совпадение
        :param contains: наоборот ищем выдачи где нет query
        :return: list с подходящими выдачами
        """
        return self._param_search("category", category, complete_match, contains)

    def get_new_list_ex(self):
        return self._new_list_ex

    def write_file(self) -> bool:
        """записываем результат в файл у которого путь path_write_file"""
        if self.path_write_file == "":
            raise "Не указан path_write_file"
        if not self._new_list_ex:
            self._new_csv.write_csv([])
            return True

        new_data_csv = [x.record_for_write_file() for x in self._new_list_ex]
        self._new_csv.write_csv(new_data_csv)
        return True



# 0 (задний фонарь на велосипед) 'Search Query'
# 1 10776176                     'PresetID'
# 2 yes                          'Active'
# 3 common                       'Kind'
# 4 empty                        'Parent'
# 5 elasticsearch (miner)        'Miner'
# 6 miner's parametr 'Miner'     "Miner's args"
# 7 presets/bucket_120           'Shard Kind'
# 8 preset=10776176              'Query'
# 9 ()                           'Category\n'
from wbxSearch.minerArgs import MinerArgs


class Extradition:
    """класс для работы с записями выгрузки"""

    def __init__(self, extradition_arr, len_extradition: tuple[int] = (10,)):
        if isinstance(extradition_arr, str):
            self.extradition_arr = extradition_arr.split("|")
        else:
            self.extradition_arr = extradition_arr

        self.len_extradition = len_extradition

        if self.check_len_extradition() is False:
            print(extradition_arr)
            raise f"Не верный формат выдачи"

        if len(self.extradition_arr) == 10:
            # стандартная длина выдачи
            self.search_query = self.extradition_arr[0]
            self.preset_id = self.extradition_arr[1]
            self.active = self.extradition_arr[2]
            self.kind = self.extradition_arr[3]
            self.parent = self.extradition_arr[4]
            self.miner = self.extradition_arr[5]
            self.miners_args = self.extradition_arr[6]
            self.miners_args_object = MinerArgs(self.miners_args, self.miner)
            self.shard_kind = self.extradition_arr[7]
            self.query = self.extradition_arr[8]
            self.category = self.extradition_arr[9]

    def check_len_extradition(self) -> bool:

        len_extradition_arr = len(self.extradition_arr)

        for value_len in self.len_extradition:
            if len_extradition_arr == value_len:
                return True
        return False

    def get_active(self):
        return True if self.active.lower() == "yes" else False

    def get_param(self, param: str) -> str:
        match param:
            case "search_query":
                return self.search_query
            case "preset_id":
                return self.preset_id
            case "active":
                return self.active
            case "kind":
                return self.kind
            case "parent":
                return self.parent
            case "miner":
                return self.miner
            case "miners_args":
                return self.miners_args
            case "shard_kind":
                return self.shard_kind
            case "query":
                return self.query
            case "category":
                return self.category
            case _:
                raise "Не правильно задан параметр"

    def record_for_write_file(self):
        return f"{self.search_query}|{self.preset_id}|{self.active}|{self.kind}|{self.parent}|{self.miner}|" \
               f"{self.miners_args_object.get_result_for_write()}|" \
               f"{self.shard_kind}|{self.query}|{self.category}".split("|")

    def __str__(self):
        return f"{self.search_query}|{self.preset_id}|{self.active}|{self.kind}|{self.parent}|{self.miner}|{self.miners_args_object.get_result_for_write()}|{self.shard_kind}|{self.query}|{self.category}"



import csv


class WorkCSV:
    def __init__(self, path_file: str = ""):
        self.path_file = path_file

    def read_csv(self, delimiter: bool = True, delimiter_symbol: str = "|") -> list:
        """читаем csv файл, отдаем инфо инфо в виде list"""
        if delimiter:
            with open(self.path_file) as f:
                reader = csv.reader(f, delimiter=delimiter_symbol, quotechar="}")
                list_info_csv = list(reader)

            return list_info_csv
        else:
            list_info_csv = []
            with open(self.path_file) as f:
                reader = csv.reader(f)
                for row in reader:
                    list_info_csv.append(row)
            return list_info_csv

    def write_csv(self, data: list, delimiter: str = "|"):
        """создаем новый csv файл, прнимает путь, название файла и  data(инфо, которую записываем в файл)"""

        with open(self.path_file, 'w') as f:
            writer = csv.writer(f, delimiter=delimiter)
            for row in data:
                writer.writerow(row)

    def selective_reading_csv(self, fragment: str = None) -> list:
        """Записываем в data(в результат выполнения функции, только те строки,
        в которым присутствует fragment)"""
        my_data = []
        with open(self.path_file) as csvfile:
            spam_reader = csv.reader(csvfile)

            for row in spam_reader:
                if fragment in (', '.join(row)):
                    my_data.append((', '.join(row)).split('|'))
        return my_data



from wbxSearch.filterArgs import FilterArgs


class MinerArgs:
    def __init__(self, str_args, miner):
        self.miner = miner
        self.str_args = str_args

        """elasticsearch"""
        self.context_subject = None
        self.query = None
        self.irrelevant_score_percent = None
        self.filter = None
        self.max_product = None
        self.boost_query = None
        self.boost_query_percent = None
        self.sorting_group_number = None
        self.search_model = None
        self.adult_filter_show_threshold = None
        self.enrich_response = None
        self.sort_version = None
        self.keep_adult = None

        self.limit = None
        self.arr_urls = []
        self.limit = None
        self.country = None
        self.timeout_between_requests = None
        self.one_by_one_join = None
        # @sort
        self.apply_self_ranker_sales = None
        self.apply_self_ranker_views = None
        self.self_ranker_views_weight = None
        self.self_ranker_sales_weight = None

        self.arr_str_args = self.str_args.split("--")

        self._flag_boost_query = "boost-query"
        self._flag_adult_filter_show_threshold = "adult-filter-show-threshold"
        self._flag_max_product = "max-product"
        self._flag_boost_query_percent = "boost-query-percent"
        self._flag_sorting_group_number = "sorting-group-number"
        self._flag_search_model = "search-model"
        self._flag_enrich_response = "enrich-response"
        self._flag_sort_version = "sort-version"
        self._flag_keep_adult = "keep-adult"

        self._flag_one_by_one_join = "--one-by-one-join"
        self._flag_limit = "limit"
        self._flag_country = "country"
        # @sort
        self._flag_apply_self_ranker_sales = "apply-self-ranker-sales"
        self._flag_apply_self_ranker_views = "apply-self-ranker-views"
        self._flag_self_ranker_views_weight = "self-ranker-views-weight"
        self._flag_self_ranker_sales_weight = "self-ranker-sales-weight"

        self._flag_context_subject = "context-subject"
        self._flag_query = "query"
        self._flag_irrelevant_score_percent = "irrelevant-score-percent"
        self._flag_filter = "filter"
        self._flag_timeout_between_requests = "timeoutBetweenRequests"

        self.parse_args()
        self.filter_object = FilterArgs(self.filter)

    def parse_args(self):
        if self._check_str_args():
            self.search_url()
            self.apply_self_ranker_sales = self._check_and_set_arg(self._flag_apply_self_ranker_sales)
            self.apply_self_ranker_views = self._check_and_set_arg(self._flag_apply_self_ranker_views)
            self.self_ranker_views_weight = self._check_and_set_arg(self._flag_self_ranker_views_weight)
            self.self_ranker_sales_weight = self._check_and_set_arg(self._flag_self_ranker_sales_weight)
            self.one_by_one_join = self._check_and_set_arg(self._flag_one_by_one_join)
            self.query = self._check_and_set_arg(self._flag_query)
            self.limit = self._check_and_set_arg(self._flag_limit)
            self.context_subject = self._check_and_set_arg(self._flag_context_subject)
            self.filter = self._check_and_set_arg(self._flag_filter)
            self.irrelevant_score_percent = self._check_and_set_arg(self._flag_irrelevant_score_percent)
            self.country = self._check_and_set_arg(self._flag_country, separator=" ")
            self.timeout_between_requests = self._check_and_set_arg(self._flag_timeout_between_requests,
                                                                    separator=" ")
            self.max_product = self._check_and_set_arg(self._flag_max_product)
            self.boost_query = self._check_and_set_arg(self._flag_boost_query)
            self.boost_query_percent = self._check_and_set_arg(self._flag_boost_query_percent)
            self.sorting_group_number = self._check_and_set_arg(self._flag_sorting_group_number)
            self.search_model = self._check_and_set_arg(self._flag_search_model)
            self.adult_filter_show_threshold = self._check_and_set_arg(self._flag_adult_filter_show_threshold)
            self.enrich_response = self._check_and_set_arg(self._flag_enrich_response)
            self.sort_version = self._check_and_set_arg(self._flag_sort_version)
            self.keep_adult = self._check_and_set_arg(self._flag_keep_adult)

    def _check_str_args(self) -> bool:
        return True if self.str_args else False

    def _get_arg(self, key, separator="=") -> str:
        """в этой функции достаем аргумент по флагу"""
        result = ""
        for arg in self.arr_str_args:
            if arg.startswith(key):
                arr_key = arg.split(separator)
                result = arr_key[1].replace('"', "")
                break
        return result

    # новое-------------------------------

    def _check_and_set_arg(self, flag_arg: str, separator: str = "="):
        if self._check_flag(flag_arg):
            try:
                return self._set_arg(flag_arg, separator)
            except IndexError:
                return self._set_arg(flag_arg, separator=" ")
        return None

    def _check_flag(self, flag_arg: str):
        return flag_arg in self.str_args

    def _set_arg(self, flag_arg: str, separator: str = "="):
        return self.del_space(self._get_arg(flag_arg, separator))

    # новое-------------------------------



    def search_url(self):
        list_str_args = self.str_args.split(" ")
        for x in list_str_args:
            if x.startswith('"http'):
                self.arr_urls.append(x)

    def construct_url_record(self):
        str_urls = " ".join(self.arr_urls)
        sort = f"{str_urls} @sort: --{self._flag_apply_self_ranker_sales}={self.apply_self_ranker_sales} --{self._flag_apply_self_ranker_views}={self.apply_self_ranker_views}"
        return sort

    def construct_url_record_for_elasticsearch(self):
        list_result = []
        if self.apply_self_ranker_sales:
            list_result.append(f'--{self._flag_apply_self_ranker_sales}={self.apply_self_ranker_sales}')
        if self.apply_self_ranker_views:
            list_result.append(f'--{self._flag_apply_self_ranker_views}={self.apply_self_ranker_views}')
        if self.self_ranker_views_weight:
            list_result.append(
                f'--{self._flag_self_ranker_views_weight}={self.self_ranker_views_weight}')
        if self.self_ranker_sales_weight:
            list_result.append(
                f'--{self._flag_self_ranker_sales_weight}={self.self_ranker_sales_weight}')
        sort = f"@sort: {' '.join(list_result)}"
        return sort

    def create_result_elasticsearch(self):
        """Собираем результат для выдачи, новый str_args, функция для майнера elasticsearch"""
        list_result = []

        if self.context_subject:
            list_result.append(f'--{self._flag_context_subject}="{self.context_subject}"')
        if self.max_product:
            list_result.append(f'--{self._flag_max_product}={self.max_product}')
        if self.query:
            list_result.append(f'--{self._flag_query}="{self.del_sort(self.query)}"')
        if self.keep_adult:
            list_result.append(f'--{self._flag_keep_adult}={self.keep_adult}')
        if self.boost_query:
            list_result.append(f'--{self._flag_boost_query}="{self.boost_query}"')
        if self.boost_query_percent:
            list_result.append(f'--{self._flag_boost_query_percent}={self.boost_query_percent}')
        if self.sorting_group_number:
            list_result.append(f'--{self._flag_sorting_group_number}={self.sorting_group_number}')
        if self.search_model:
            list_result.append(f'--{self._flag_search_model}={self.search_model}')
        if self.sort_version:
            list_result.append(f'--{self._flag_sort_version}={self.sort_version}')
        if self.enrich_response:
            list_result.append(f'--{self._flag_enrich_response}={self.enrich_response}')
        if self.adult_filter_show_threshold:
            list_result.append(f'--{self._flag_adult_filter_show_threshold}'
                               f'={self.adult_filter_show_threshold}')
        if self.irrelevant_score_percent:
            list_result.append(
                f"--{self._flag_irrelevant_score_percent}={self.irrelevant_score_percent}")
        if self.filter:
            list_result.append(f'--{self._flag_filter}="{self.filter_object.get_result_for_write()}"')
        if self.apply_self_ranker_views or self.apply_self_ranker_sales or self.self_ranker_sales_weight or \
                self.self_ranker_views_weight:
            list_result.append(self.construct_url_record_for_elasticsearch())
        return " ".join(list_result)

    def get_result_for_write(self):
        if self.miner == "elasticsearch":
            result = self.create_result_elasticsearch()
            return result
        result = self.str_args
        return result

    def del_space(self, arg):
        if arg[-1:] == " ":
            return arg[:-1]
        return arg

    def del_sort(self, arg):
        if arg[-7:] == " @sort:":
            return arg[:-7]
        return arg

    def __str__(self):
        return self.str_args



#!usr/bin/env python

from io import open
from setuptools import setup

"""
:authors: ZhuravlevDmi
:license: Apache License, version 2.0, see LICENSE file
"""

version = "0.1.1"

long_description = "..."

setup(
    name="wbxSearch",
    version=version,
    author="ZhuravlevDmi",
    author_email="Dima15129@gmail.com",

    description=(u"Удобный инструмент для работы с выдачами"),
    long_description=long_description,
    url="https://github.com/ZhuravlevDmi/wbxSearch",
    download_url="https://github.com/ZhuravlevDmi/wbxSearch/archive/v{}.zip".format(version),

    license="Apache License, version 2.0, see LICENSE file",
    packages=["wbxSearch"],
    install_requires=[],
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        # "Programming Language :: Python :: 3",
        # "Programming Language :: Python :: 3.5",
        # "Programming Language :: Python :: 3.6",
        # "Programming Language :: Python :: 3.7",
        # "Programming Language :: Python :: 3.8",
        # "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Programming Language :: Python :: Implementation :: CPython",
    ]

)



"""
:authors: ZhuravlevDmi
:license: Apache License, version 2.0, see LICENSE file
"""

from .classCSV import WorkCSV
from .extradition import Extradition
from .filterArgs import FilterArgs
from .minerArgs import MinerArgs
from .WorkExtradition import WorkExtradition

__author__ = "ZhuravlevDmi"
__version__ = "0.1.1"
__email__ = "Dima15129@gmail.com"
